VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Session"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"No"
Attribute VB_Ext_KEY = "Member0" ,"Players"
'##CD Session class is representation of a physical meeting of Players. Session classes control the transmition of data between Player objects associated with it.
Attribute VB_HelpID=2023
Attribute VB_Description="Session class is representation of a physical meeting of Players. Session classes control the transmition of data between Player objects associated with it."
Option Explicit

Public Key As String

'local variable(s) to hold property value(s)
Private mvarSessionData As DirectPlaySessionData
Private mvarCurrentPlayers As Long
Private mvarFlags As Long
Private mvarGuidApplication As String
Private mvarGuidInstance As String
Private mvarMaxPlayers As Long
Private mvarName_ As String
Private mvarPassword As String
Private mvarUserDefined1 As Long
Private mvarUserDefined2 As Long
Private mvarUserDefined3 As Long
Private mvarUserDefined4 As Long
Private mvarPlayers As Players
Private mvarCapabilities As DPCAPS
Private mvarCapabilitiesGuaranteed As DPCAPS
Private mvarParent As Sessions
Private mvarJoined As Boolean

Public Enum meDeliverTypes
'##EN Deliver types when sending messages.
    dtDefaultDelivery = 0
    '##EM dtDefaultDelivery Default method.
    dtEncryptedDelivery = 64
    '##EM dtEncryptedDelivery Sends the message encrypted. This can only be done in a secure session. This flag can only be used if the DPSEND_GUARANTEED flag is also set. The message will be sent as a DPSYS_SECUREMESSAGE system message.
    dtGuaranteedDelivery = 1
    '##EM dtGuaranteedDelivery Sends the message by using a guaranteed method of delivery.
    dtSignedDelivery = 32
    '##EM dtSignedDelivery Sends the message with a digital signature. This can only be done in a secure session. This flag can only be used if the DPSEND_SIGNED flag is also set. The message will be sent as a DPSYS_SECUREMESSAGE system message.
    dtAsyncDelivery = 512

    dtNoSendCompleteMessageDelivery = 1024

End Enum

Attribute Key.VB_VarHelpID = 2026
Public Function Join(ByVal LongName As String, ByVal ShortName As String, _
                                      Optional ByVal Flags As CONST_DPPLAYERFLAGS = DPPLAYER_DEFAULT) As Boolean
Attribute Join.VB_HelpID = 2027
Attribute Join.VB_Description = "Joins the Session."
'##BD Joins the Session.

'##PD LongName Formal name of the new Player.
'##PD ShortName Friendly name of the new Player.
'##PD Flags Flags (Options) of the new Player.
'##RD Returns True if Session was joined False if an error occured.
    Dim PlayerID As Long, Player As Player
    On Error GoTo Handler

    If LongName = "" Then LongName = " "
    If ShortName = "" Then ShortName = " "

    'Join this session.
    Call MEngine.DirectPlayObject.Open(SessionData, DPOPEN_JOIN)
    mvarJoined = True

    'Create the player and add him to the collection.
    PlayerID = MEngine.DirectPlayObject.CreatePlayer(ShortName, LongName, NotificationID, Flags)

    Call RefreshPlayers 'Refresh the classes.
    Call Parent.SetActiveSession(Me) 'Set the active session

    Call mvarPlayers.Item(CStr(PlayerID)).SetMyPlayerFlag 'Make that player object to say this is me.
    Call MEngine.DirectPlayObject.GetSessionDesc(mvarSessionData) 'Refresh the session data.
    Call SetInformation(mvarSessionData, Parent)
    Call mvarPlayers.SetMyPlayer(mvarPlayers.Item(CStr(PlayerID))) 'Set the MyPlayer object.

    'Refresh the capabilities
    Call MEngine.DirectPlayObject.GetCaps(mvarCapabilities, DPGETCAPS_DEFAULT)
    Call MEngine.DirectPlayObject.GetCaps(mvarCapabilitiesGuaranteed, DPGETCAPS_GUARANTEED)

    Join = True 'Set flags

    Call MEngine.FireEvent(6, , , , mvarPlayers.Item(CStr(PlayerID))) 'Fire MyPlayerAdded event
    Call MEngine.FireEvent(5, , , Me) 'Fire SessionJoined event

ExitIt:
    Set Player = Nothing
    Exit Function

Handler:
    If Err = DPERR_INVALIDPARAMS Then
        Call MEngine.FireErrorEvent(Err.Number, "One or more of the parameters passed to the method are invalid.", "Session Join Function")
    ElseIf DPERR_ACCESSDENIED Then
        Call MEngine.FireErrorEvent(Err.Number, "The session is full, or an incorrect password was supplied.", "Session Join Function")
    ElseIf DPERR_ALREADYINITIALIZED Then
        Call MEngine.FireErrorEvent(Err.Number, "This object is already initialized.", "Session Join Function")
    ElseIf DPERR_AUTHENTICATIONFAILED Then
        Call MEngine.FireErrorEvent(Err.Number, "The password or credentials supplied could not be authenticated.", "Session Join Function")
    ElseIf DPERR_CANNOTCREATESERVER Then
        Call MEngine.FireErrorEvent(Err.Number, "The server cannot be created for the new session. ", "Session Join Function")
    ElseIf DPERR_CANTCREATEPLAYER Then
        Call MEngine.FireErrorEvent(Err.Number, "A new player cannot be created.", "Session Join Function")
    ElseIf DPERR_CANTLOADCAPI Then
        Call MEngine.FireErrorEvent(Err.Number, "No credentials were supplied and the CryptoAPI package (CAPI) to use for cryptography services cannot be loaded.", "Session Join Function")
    ElseIf DPERR_CANTLOADSECURITYPACKAGE Then
        Call MEngine.FireErrorEvent(Err.Number, "The software security package cannot be loaded.", "Session Join Function")
    ElseIf DPERR_CANTLOADSSPI Then
        Call MEngine.FireErrorEvent(Err.Number, "No credentials were supplied, and the Security Support Provider Interface (SSPI) that will prompt for credentials cannot be loaded.", "Session Join Function")
    ElseIf DPERR_CONNECTING Then
        Call MEngine.FireErrorEvent(Err.Number, "The method is in the process of connecting to the network. The application should keep using the method until it returns DP_OK, indicating successful completion, or until it returns a different error.", "Session Join Function")
    ElseIf DPERR_CONNECTIONLOST Then
        Call MEngine.FireErrorEvent(Err.Number, "The service provider connection was reset while data was being sent.", "Session Join Function")
    ElseIf DPERR_ENCRYPTIONFAILED Then
        Call MEngine.FireErrorEvent(Err.Number, "The requested information could not be digitally encrypted. Encryption is used for message privacy. This error is only relevant in a secure session.", "Session Join Function")
    ElseIf DPERR_INVALIDFLAGS Then
        Call MEngine.FireErrorEvent(Err.Number, "The flags passed to this method are invalid.", "Session Join Function")
    ElseIf DPERR_INVALIDPASSWORD Then
        Call MEngine.FireErrorEvent(Err.Number, "An invalid password was supplied when attempting to join a session that requires a password.", "Session Join Function")
    ElseIf DPERR_LOGONDENIED Then
        Call MEngine.FireErrorEvent(Err.Number, "The session could not be opened because credentials are required, and either no credentials were supplied, or the credentials were invalid.", "Session Join Function")
    ElseIf DPERR_NOCONNECTION Then
        Call MEngine.FireErrorEvent(Err.Number, "No communication link was established.", "Session Join Function")
    ElseIf DPERR_NONEWPLAYERS Then
        Call MEngine.FireErrorEvent(Err.Number, "The session is not accepting any new players.", "Session Join Function")
    ElseIf DPERR_NOSESSIONS Then
        Call MEngine.FireErrorEvent(Err.Number, "There are no sessions for which this method can be called.", "Session Join Function")
    ElseIf DPERR_SIGNFAILED Then
        Call MEngine.FireErrorEvent(Err.Number, "The requested information could not be digitally signed. Digital signatures are used to establish the authenticity of messages.", "Session Join Function")
    ElseIf DPERR_TIMEOUT Then
        Call MEngine.FireErrorEvent(Err.Number, "The operation could not be completed in the specified time.", "Session Join Function")
    ElseIf DPERR_UNINITIALIZED Then
        Call MEngine.FireErrorEvent(Err.Number, "The requested object has not been initialized.", "Session Join Function")
    ElseIf DPERR_USERCANCEL Then
        Call MEngine.FireErrorEvent(Err.Number, "Can be returned in two ways. 1) The user canceled the connection process during a call to the DirectPlay4.Open method. 2) The user clicked Cancel in one of the DirectPlay service provider dialog boxes during a call to DirectPlay4.GetDPEnumSessions.", "Session Join Function")
    ElseIf DPERR_CANTADDPLAYER Then
        Call MEngine.FireErrorEvent(Err.Number, "The player cannot be added to the session.", "Session Join Function")
    ElseIf DPERR_CANTCREATEPLAYER Then
        Call MEngine.FireErrorEvent(Err.Number, "A new player cannot be created.", "Session Join Function")
    Else
        Call MEngine.FireErrorEvent(Err.Number, Err.Description, "Session Join Function")
    End If

    Join = False:   mvarJoined = False
    GoTo ExitIt
End Function

Public Function JoinSecured(ByVal LongName As String, ByVal ShortName As String, Security As DPSECURITYDESC, _
                                      Credentials As DPCREDENTIALS, Optional ByVal Flags As CONST_DPPLAYERFLAGS = _
                                      DPPLAYER_DEFAULT) As Boolean
Attribute JoinSecured.VB_HelpID = 2028
Attribute JoinSecured.VB_Description = "Joins the secured Session."
'##BD Joins the secured Session.

'##PD LongName Formal name of the new Player.
'##PD ShortName Friendly name of the new Player.
'##PD Security DPSECURITYDESC type containing the security package to use.
'##PD Credentials DPCREDENTIALS type containing the logon name, password, and domain to be authenticated on the server.
'##PD Flags Flags (Options) of the new Player.
'##RD Returns True if Session was joined False if an error occured.
    Dim PlayerID As Long, Player As Player
    On Error GoTo Handler

    'Join this session.
    Call MEngine.DirectPlayObject.SecureOpen(SessionData, DPOPEN_JOIN, Security, Credentials)
    mvarJoined = True

    'Create the player and add him to the collection.
    PlayerID = MEngine.DirectPlayObject.CreatePlayer(ShortName, LongName, NotificationID, Flags)

    Call RefreshPlayers 'Refresh the classes.
    Call Parent.SetActiveSession(Me) 'Set the active session

    Call mvarPlayers.Item(CStr(PlayerID)).SetMyPlayerFlag 'Make that player object to say this is me.
    Call MEngine.DirectPlayObject.GetSessionDesc(mvarSessionData) 'Refresh the session data.
    Call SetInformation(mvarSessionData, Parent)
    Call mvarPlayers.SetMyPlayer(mvarPlayers.Item(CStr(PlayerID))) 'Set the MyPlayer object.

    'Refresh the capabilities
    Call MEngine.DirectPlayObject.GetCaps(mvarCapabilities, DPGETCAPS_DEFAULT)
    Call MEngine.DirectPlayObject.GetCaps(mvarCapabilitiesGuaranteed, DPGETCAPS_GUARANTEED)

    JoinSecured = True 'Set flags

    Call MEngine.FireEvent(6, , , , mvarPlayers.Item(CStr(PlayerID))) 'Fire MyPlayerAdded event
    Call MEngine.FireEvent(5, , , Me) 'Fire SessionJoined event

ExitIt:
    Set Player = Nothing
    Exit Function

Handler:
    If Err = DPERR_INVALIDPARAMS Then
        Call MEngine.FireErrorEvent(Err.Number, "One or more of the parameters passed to the method are invalid.", "Session JoinSecured Function")
    ElseIf DPERR_ACCESSDENIED Then
        Call MEngine.FireErrorEvent(Err.Number, "The session is full, or an incorrect password was supplied.", "Session JoinSecured Function")
    ElseIf DPERR_ALREADYINITIALIZED Then
        Call MEngine.FireErrorEvent(Err.Number, "This object is already initialized.", "Session JoinSecured Function")
    ElseIf DPERR_AUTHENTICATIONFAILED Then
        Call MEngine.FireErrorEvent(Err.Number, "The password or credentials supplied could not be authenticated.", "Session JoinSecured Function")
    ElseIf DPERR_CANNOTCREATESERVER Then
        Call MEngine.FireErrorEvent(Err.Number, "The server cannot be created for the new session. ", "Session JoinSecured Function")
    ElseIf DPERR_CANTCREATEPLAYER Then
        Call MEngine.FireErrorEvent(Err.Number, "A new player cannot be created.", "Session JoinSecured Function")
    ElseIf DPERR_CANTLOADCAPI Then
        Call MEngine.FireErrorEvent(Err.Number, "No credentials were supplied and the CryptoAPI package (CAPI) to use for cryptography services cannot be loaded.", "Session JoinSecured Function")
    ElseIf DPERR_CANTLOADSECURITYPACKAGE Then
        Call MEngine.FireErrorEvent(Err.Number, "The software security package cannot be loaded.", "Session JoinSecured Function")
    ElseIf DPERR_CANTLOADSSPI Then
        Call MEngine.FireErrorEvent(Err.Number, "No credentials were supplied, and the Security Support Provider Interface (SSPI) that will prompt for credentials cannot be loaded.", "Session JoinSecured Function")
    ElseIf DPERR_CONNECTING Then
        Call MEngine.FireErrorEvent(Err.Number, "The method is in the process of connecting to the network. The application should keep using the method until it returns DP_OK, indicating successful completion, or until it returns a different error.", "Session JoinSecured Function")
    ElseIf DPERR_CONNECTIONLOST Then
        Call MEngine.FireErrorEvent(Err.Number, "The service provider connection was reset while data was being sent.", "Session JoinSecured Function")
    ElseIf DPERR_ENCRYPTIONFAILED Then
        Call MEngine.FireErrorEvent(Err.Number, "The requested information could not be digitally encrypted. Encryption is used for message privacy. This error is only relevant in a secure session.", "Session JoinSecured Function")
    ElseIf DPERR_INVALIDFLAGS Then
        Call MEngine.FireErrorEvent(Err.Number, "The flags passed to this method are invalid.", "Session JoinSecured Function")
    ElseIf DPERR_INVALIDPASSWORD Then
        Call MEngine.FireErrorEvent(Err.Number, "An invalid password was supplied when attempting to JoinSecured a session that requires a password.", "Session JoinSecured Function")
    ElseIf DPERR_LOGONDENIED Then
        Call MEngine.FireErrorEvent(Err.Number, "The session could not be opened because credentials are required, and either no credentials were supplied, or the credentials were invalid.", "Session JoinSecured Function")
    ElseIf DPERR_NOCONNECTION Then
        Call MEngine.FireErrorEvent(Err.Number, "No communication link was established.", "Session JoinSecured Function")
    ElseIf DPERR_NONEWPLAYERS Then
        Call MEngine.FireErrorEvent(Err.Number, "The session is not accepting any new players.", "Session JoinSecured Function")
    ElseIf DPERR_NOSESSIONS Then
        Call MEngine.FireErrorEvent(Err.Number, "There are no sessions for which this method can be called.", "Session JoinSecured Function")
    ElseIf DPERR_SIGNFAILED Then
        Call MEngine.FireErrorEvent(Err.Number, "The requested information could not be digitally signed. Digital signatures are used to establish the authenticity of messages.", "Session JoinSecured Function")
    ElseIf DPERR_TIMEOUT Then
        Call MEngine.FireErrorEvent(Err.Number, "The operation could not be completed in the specified time.", "Session JoinSecured Function")
    ElseIf DPERR_UNINITIALIZED Then
        Call MEngine.FireErrorEvent(Err.Number, "The requested object has not been initialized.", "Session JoinSecured Function")
    ElseIf DPERR_USERCANCEL Then
        Call MEngine.FireErrorEvent(Err.Number, "Can be returned in two ways. 1) The user canceled the connection process during a call to the DirectPlay4.Open method. 2) The user clicked Cancel in one of the DirectPlay service provider dialog boxes during a call to DirectPlay4.GetDPEnumSessions.", "Session JoinSecured Function")
    ElseIf DPERR_CANTADDPLAYER Then
        Call MEngine.FireErrorEvent(Err.Number, "The player cannot be added to the session.", "Session JoinSecured Function")
    ElseIf DPERR_CANTCREATEPLAYER Then
        Call MEngine.FireErrorEvent(Err.Number, "A new player cannot be created.", "Session JoinSecured Function")
    Else
        Call MEngine.FireErrorEvent(Err.Number, Err.Description, "Session JoinSecured Function")
    End If

    JoinSecured = False:   mvarJoined = False
    GoTo ExitIt
End Function


Public Property Get Players() As Players
Attribute Players.VB_HelpID = 2029
Attribute Players.VB_Description = "Collection of Player objects that represents each Player currently associated with the Session."
'##BD Collection of Player objects that represents each Player currently associated with the Session.
    Set Players = mvarPlayers
End Property



Public Function RefreshPlayers() As Boolean
Attribute RefreshPlayers.VB_HelpID = 2030
Attribute RefreshPlayers.VB_Description = "Updates the Players collection."
'##BD Updates the Players collection.
    Dim DPEnumPlayers As DirectPlayEnumPlayers
    Dim lCount As Long, Player As Player
    On Error GoTo Handler

    Call mvarPlayers.Clear

    'Enumerate the players.
    If mvarJoined = False Then 'If we haven't joined this session we need to use the GUID to get the players.
        Set DPEnumPlayers = MEngine.DirectPlayObject.GetDPEnumPlayers(GuidInstance, DPENUMPLAYERS_ALL _
                                                                                                                                                            Or DPENUMPLAYERS_SESSION)
    Else 'We have joined the session so we don't need the GUID to get the players.
        Set DPEnumPlayers = MEngine.DirectPlayObject.GetDPEnumPlayers("", DPENUMPLAYERS_ALL)
    End If

    'Load the collection with the returned players.
    For lCount = 1 To DPEnumPlayers.GetCount
        Set Player = mvarPlayers.Add(CStr(lCount), DPEnumPlayers.GetDPID(lCount))
        Call Player.SetInformation(DPEnumPlayers.GetDPID(lCount), DPEnumPlayers.GetFlags(lCount), DPEnumPlayers.GetLongName(lCount) _
                                                        , DPEnumPlayers.GetShortName(lCount), DPEnumPlayers.GetType(lCount))
        If Not mvarPlayers.MyPlayer Is Nothing Then 'We need to set the Myplayer flag for our player.
            If Player.DPID = mvarPlayers.MyPlayer.DPID Then Player.SetMyPlayerFlag
        End If
        Call MEngine.FireEvent(3, , , Me, Player)
    Next lCount
    RefreshPlayers = True
    Exit Function

Handler:
    If Err = DPERR_ACCESSDENIED Then
        Call MEngine.FireErrorEvent(Err.Number, "The session is full, or an incorrect password was supplied.", "ServiceProvider RefreshSessions Function")
    ElseIf Err = DPERR_CONNECTIONLOST Then
        Call MEngine.FireErrorEvent(Err.Number, "The service provider connection was reset while data was being sent.", "ServiceProvider RefreshSessions Function")
    ElseIf Err = DPERR_INVALIDPARAMS Then
        Call MEngine.FireErrorEvent(Err.Number, "One or more of the parameters passed to the method are invalid.", "ServiceProvider RefreshSessions Function")
    ElseIf Err = DPERR_NOSESSIONS Then
        Call MEngine.FireErrorEvent(Err.Number, "There are no sessions for which this method can be called.", "ServiceProvider RefreshSessions Function")
    ElseIf Err = DPERR_UNAVAILABLE Then
        Call MEngine.FireErrorEvent(Err.Number, "The requested function is not available at this time.", "ServiceProvider RefreshSessions Function")
    Else
        Call MEngine.FireErrorEvent(Err.Number, Err.Description, "Session RefreshPlayers Function")
    End If
    RefreshPlayers = False
End Function
'Public Function SendChatMessage(ByVal Msg As String, Optional ByVal ToPlayer As Long = DPID_ALLPLAYERS, _
'                                                                    Optional ByVal Flags As meDeliverTypes = [Default Delivery]) As Boolean
'    On Error GoTo Handler
'
'    Call MEngine.DirectPlayObject.SendChatMessage(mvarPlayers.MyPlayer.DPID, ToPlayer, Flags, Msg)
'    SendChatMessage = True
'    Exit Function
'
'Handler:
'    If Err = DPERR_ACCESSDENIED Then
'        Call MEngine.FireErrorEvent(Err.Number, "The session is full, or an incorrect password was supplied.", "Session SendChatMessage Function")
'    ElseIf Err = DPERR_CONNECTIONLOST Then
'        Call MEngine.FireErrorEvent(Err.Number, "The service provider connection was reset while data was being sent.", "Session SendChatMessage Function")
'    ElseIf Err = DPERR_INVALIDPARAMS Then
'        Call MEngine.FireErrorEvent(Err.Number, "One or more of the parameters passed to the method are invalid.", "Session SendChatMessage Function")
'    ElseIf Err = DPERR_INVALIDFLAGS Then
'        Call MEngine.FireErrorEvent(Err.Number, "The flags passed to this method are invalid.", "Session SendChatMessage Function")
'    ElseIf Err = DPERR_INVALIDPLAYER Then
'        Call MEngine.FireErrorEvent(Err.Number, "The player ID is not recognized as a valid player ID for this game session.", "Session SendChatMessage Function")
'    Else
'        Call MEngine.FireErrorEvent(Err.Number, Err.Description, "Session SendChatMessage Function")
'    End If
'    SendChatMessage = False
'End Function

Public Function Send(ByVal DPMsg As DirectPlayMessage, Optional ByVal ToPlayer As Long = DPID_ALLPLAYERS, _
                                                                    Optional ByVal Flags As meDeliverTypes = dtDefaultDelivery) As Boolean
Attribute Send.VB_HelpID = 2031
Attribute Send.VB_Description = "Sends a DirectPlayMessage object to another Player or Group. Use this function if you wish to create your own DirectPlayMessage object and send it to another player or group."
'##BD Sends a DirectPlayMessage object to another Player or Group. Use this function if you wish to create your own DirectPlayMessage object and send it to another player or group.

'##PD DPMsg DirectPlayMessage object containing information to send.
'##PD ToPlayer Player to send the message too. Leaving this value to its default DPID_ALLPLAYERS constant value will send the message to everyone connected to the session.
'##PD Flags Deliver flags.
'##RD Returns True if the message was sent correctly or False if an error occured.
    On Error GoTo Handler

    Call MEngine.DirectPlayObject.Send(mvarPlayers.MyPlayer.DPID, ToPlayer, Flags, DPMsg)
    Call MEngine.FireEvent(7, , , , , DPMsg)
    Send = True
    Exit Function

Handler:
    If Err = DPERR_ACCESSDENIED Then
        Call MEngine.FireErrorEvent(Err.Number, "The session is full, or an incorrect password was supplied.", "Session Send Function")
    ElseIf Err = DPERR_CONNECTIONLOST Then
        Call MEngine.FireErrorEvent(Err.Number, "The service provider connection was reset while data was being sent.", "Session Send Function")
    ElseIf Err = DPERR_INVALIDPARAMS Then
        Call MEngine.FireErrorEvent(Err.Number, "One or more of the parameters passed to the method are invalid.", "Session Send Function")
    ElseIf Err = DPERR_NOTLOGGEDIN Then
        Call MEngine.FireErrorEvent(Err.Number, "An action cannot be performed because a player or client application is not logged on. Returned by the DirectPlay4.Send method when the client application tries to send a secure message without being logged on.", "Session Send Function")
    ElseIf Err = DPERR_INVALIDPLAYER Then
        Call MEngine.FireErrorEvent(Err.Number, "The player ID is not recognized as a valid player ID for this game session.", "Session Send Function")
    ElseIf Err = DPERR_BUSY Then
        Call MEngine.FireErrorEvent(Err.Number, "A message cannot be sent because the transmission medium is busy.", "Session Send Function")
    ElseIf Err = DPERR_SENDTOOBIG Then
        Call MEngine.FireErrorEvent(Err.Number, "The message being sent by the DirectPlay4.Send method is too large.", "Session Send Function")
    Else
        Call MEngine.FireErrorEvent(Err.Number, Err.Description, "Session Send Function")
    End If
    Send = False
End Function

Public Function SendString(ByVal Message As String, ByVal Identifier As Variant, Optional ByVal ToPlayer As Long = DPID_ALLPLAYERS, _
                                                                    Optional ByVal Flags As meDeliverTypes = dtDefaultDelivery) As Boolean
Attribute SendString.VB_HelpID = 2212
Attribute SendString.VB_Description = "Sends a string message to another Player or Group."
'##BD Sends a string message to another Player or Group.

'##PD Message Information to send.
'##PD Identifier Message header. Message header allows the reciever to determine what kind of message it is. Each different type of _
                            application defined message should have a unique header (identifier). Identifier value should be one of these types _
                            (Byte, Double, Long, Short, Single, String). If another value is needed use the "Send" method.
'##PD ToPlayer Player to send the message too. Leaving this value to its default DPID_ALLPLAYERS constant value will send the message to everyone connected to the session.
'##PD Flags Deliver flags.
'##RD Returns True if the message was sent correctly or False if an error occured.
    Dim DPMsg As DirectPlayMessage
    On Error GoTo Handler

    Set DPMsg = MEngine.DirectPlayObject.CreateMessage 'Create the message object.
    With DPMsg
        'First we detemine what type of variable the identifier is so we can write it to the message object right.
        If VarType(Identifier) = vbByte Then
            Call .WriteByte(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbDouble Then
            Call .WriteDouble(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbLong Then
            Call .WriteLong(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbInteger Then
            Call .WriteLong(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbSingle Then
            Call .WriteSingle(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbString Then
            Call .WriteSingle(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        End If
        Call .WriteString(Message) 'Write the string value
    End With

    Call MEngine.DirectPlayObject.Send(mvarPlayers.MyPlayer.DPID, ToPlayer, Flags, DPMsg)
    Call MEngine.FireEvent(7, , , , , DPMsg)
    SendString = True

ExitIt:
    Set DPMsg = Nothing
    Exit Function

Handler:
    If Err = DPERR_ACCESSDENIED Then
        Call MEngine.FireErrorEvent(Err.Number, "The session is full, or an incorrect password was supplied.", "Session SendString Function")
    ElseIf Err = DPERR_CONNECTIONLOST Then
        Call MEngine.FireErrorEvent(Err.Number, "The service provider connection was reset while data was being sent.", "Session SendString Function")
    ElseIf Err = DPERR_INVALIDPARAMS Then
        Call MEngine.FireErrorEvent(Err.Number, "One or more of the parameters passed to the method are invalid.", "Session SendString Function")
    ElseIf Err = DPERR_NOTLOGGEDIN Then
        Call MEngine.FireErrorEvent(Err.Number, "An action cannot be performed because a player or client application is not logged on. Returned by the DirectPlay4.Send method when the client application tries to send a secure message without being logged on.", "Session SendString Function")
    ElseIf Err = DPERR_INVALIDPLAYER Then
        Call MEngine.FireErrorEvent(Err.Number, "The player ID is not recognized as a valid player ID for this game session.", "Session SendString Function")
    ElseIf Err = DPERR_BUSY Then
        Call MEngine.FireErrorEvent(Err.Number, "A message cannot be sent because the transmission medium is busy.", "Session SendString Function")
    ElseIf Err = DPERR_SENDTOOBIG Then
        Call MEngine.FireErrorEvent(Err.Number, "The message being sent by the DirectPlay4.Send method is too large.", "Session SendString Function")
    Else
        Call MEngine.FireErrorEvent(Err.Number, Err.Description, "Session SendString Function")
    End If
    SendString = False
    GoTo ExitIt
End Function

Public Function SendByte(ByVal Message As Byte, ByVal Identifier As Variant, Optional ByVal ToPlayer As Long = DPID_ALLPLAYERS, _
                                                                    Optional ByVal Flags As meDeliverTypes = dtDefaultDelivery) As Boolean
Attribute SendByte.VB_HelpID = 2213
Attribute SendByte.VB_Description = "Sends a byte message to another Player or Group."
'##BD Sends a byte message to another Player or Group.

'##PD Message Information to send.
'##PD Identifier Message header. Message header allows the reciever to determine what kind of message it is. Each different type of _
                            application defined message should have a unique header (identifier). Identifier value should be one of these types _
                            (Byte, Double, Long, Short, Single, String). If another value is needed use the "Send" method.
'##PD ToPlayer Player to send the message too. Leaving this value to its default DPID_ALLPLAYERS constant value will send the message to everyone connected to the session.
'##PD Flags Deliver flags.
'##RD Returns True if the message was sent correctly or False if an error occured.
    Dim DPMsg As DirectPlayMessage
    On Error GoTo Handler

    Set DPMsg = MEngine.DirectPlayObject.CreateMessage 'Create the message object.
    With DPMsg
        'First we detemine what type of variable the identifier is so we can write it to the message object right.
        If VarType(Identifier) = vbByte Then
            Call .WriteByte(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbDouble Then
            Call .WriteDouble(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbLong Then
            Call .WriteLong(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbInteger Then
            Call .WriteLong(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbSingle Then
            Call .WriteSingle(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbString Then
            Call .WriteSingle(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        End If
        Call .WriteByte(Message)   'Write the byte value.
    End With

    Call MEngine.DirectPlayObject.Send(mvarPlayers.MyPlayer.DPID, ToPlayer, Flags, DPMsg)
    Call MEngine.FireEvent(7, , , , , DPMsg)
    SendByte = True

ExitIt:
    Set DPMsg = Nothing
    Exit Function

Handler:
    If Err = DPERR_ACCESSDENIED Then
        Call MEngine.FireErrorEvent(Err.Number, "The session is full, or an incorrect password was supplied.", "Session SendByte Function")
    ElseIf Err = DPERR_CONNECTIONLOST Then
        Call MEngine.FireErrorEvent(Err.Number, "The service provider connection was reset while data was being sent.", "Session SendByte Function")
    ElseIf Err = DPERR_INVALIDPARAMS Then
        Call MEngine.FireErrorEvent(Err.Number, "One or more of the parameters passed to the method are invalid.", "Session SendByte Function")
    ElseIf Err = DPERR_NOTLOGGEDIN Then
        Call MEngine.FireErrorEvent(Err.Number, "An action cannot be performed because a player or client application is not logged on. Returned by the DirectPlay4.Send method when the client application tries to send a secure message without being logged on.", "Session SendByte Function")
    ElseIf Err = DPERR_INVALIDPLAYER Then
        Call MEngine.FireErrorEvent(Err.Number, "The player ID is not recognized as a valid player ID for this game session.", "Session SendByte Function")
    ElseIf Err = DPERR_BUSY Then
        Call MEngine.FireErrorEvent(Err.Number, "A message cannot be sent because the transmission medium is busy.", "Session SendByte Function")
    ElseIf Err = DPERR_SENDTOOBIG Then
        Call MEngine.FireErrorEvent(Err.Number, "The message being sent by the DirectPlay4.Send method is too large.", "Session SendByte Function")
    Else
        Call MEngine.FireErrorEvent(Err.Number, Err.Description, "Session SendByte Function")
    End If
    SendByte = False
    GoTo ExitIt
End Function

Public Function SendLong(ByVal Message As Long, ByVal Identifier As Variant, Optional ByVal ToPlayer As Long = DPID_ALLPLAYERS, _
                                                                    Optional ByVal Flags As meDeliverTypes = dtDefaultDelivery) As Boolean
Attribute SendLong.VB_HelpID = 2214
Attribute SendLong.VB_Description = "Sends a long message to another Player or Group."
'##BD Sends a long message to another Player or Group.

'##PD Message Information to send.
'##PD Identifier Message header. Message header allows the reciever to determine what kind of message it is. Each different type of _
                            application defined message should have a unique header (identifier). Identifier value should be one of these types _
                            (Byte, Double, Long, Short, Single, String). If another value is needed use the "Send" method.
'##PD ToPlayer Player to send the message too. Leaving this value to its default DPID_ALLPLAYERS constant value will send the message to everyone connected to the session.
'##PD Flags Deliver flags.
'##RD Returns True if the message was sent correctly or False if an error occured.
    Dim DPMsg As DirectPlayMessage
    On Error GoTo Handler

    Set DPMsg = MEngine.DirectPlayObject.CreateMessage 'Create the message object.
    With DPMsg
        'First we detemine what type of variable the identifier is so we can write it to the message object right.
        If VarType(Identifier) = vbByte Then
            Call .WriteByte(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbDouble Then
            Call .WriteDouble(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbLong Then
            Call .WriteLong(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbInteger Then
            Call .WriteLong(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbSingle Then
            Call .WriteSingle(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbString Then
            Call .WriteSingle(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        End If
        Call .WriteLong(Message)   'Write the long value.
    End With

    Call MEngine.DirectPlayObject.Send(mvarPlayers.MyPlayer.DPID, ToPlayer, Flags, DPMsg)
    Call MEngine.FireEvent(7, , , , , DPMsg)
    SendLong = True

ExitIt:
    Set DPMsg = Nothing
    Exit Function

Handler:
    If Err = DPERR_ACCESSDENIED Then
        Call MEngine.FireErrorEvent(Err.Number, "The session is full, or an incorrect password was supplied.", "Session SendLong Function")
    ElseIf Err = DPERR_CONNECTIONLOST Then
        Call MEngine.FireErrorEvent(Err.Number, "The service provider connection was reset while data was being sent.", "Session SendLong Function")
    ElseIf Err = DPERR_INVALIDPARAMS Then
        Call MEngine.FireErrorEvent(Err.Number, "One or more of the parameters passed to the method are invalid.", "Session SendLong Function")
    ElseIf Err = DPERR_NOTLOGGEDIN Then
        Call MEngine.FireErrorEvent(Err.Number, "An action cannot be performed because a player or client application is not logged on. Returned by the DirectPlay4.Send method when the client application tries to send a secure message without being logged on.", "Session SendLong Function")
    ElseIf Err = DPERR_INVALIDPLAYER Then
        Call MEngine.FireErrorEvent(Err.Number, "The player ID is not recognized as a valid player ID for this game session.", "Session SendLong Function")
    ElseIf Err = DPERR_BUSY Then
        Call MEngine.FireErrorEvent(Err.Number, "A message cannot be sent because the transmission medium is busy.", "Session SendLong Function")
    ElseIf Err = DPERR_SENDTOOBIG Then
        Call MEngine.FireErrorEvent(Err.Number, "The message being sent by the DirectPlay4.Send method is too large.", "Session SendLong Function")
    Else
        Call MEngine.FireErrorEvent(Err.Number, Err.Description, "Session SendLong Function")
    End If
    SendLong = False
    GoTo ExitIt
End Function

Public Function SendDouble(ByVal Message As Long, ByVal Identifier As Variant, Optional ByVal ToPlayer As Long = DPID_ALLPLAYERS, _
                                                                    Optional ByVal Flags As meDeliverTypes = dtDefaultDelivery) As Boolean
Attribute SendDouble.VB_HelpID = 2215
Attribute SendDouble.VB_Description = "Sends a double message to another Player or Group."
'##BD Sends a double message to another Player or Group.

'##PD Message Information to send.
'##PD Identifier Message header. Message header allows the reciever to determine what kind of message it is. Each different type of _
                            application defined message should have a unique header (identifier). Identifier value should be one of these types _
                            (Byte, Double, Long, Short, Single, String). If another value is needed use the "Send" method.
'##PD ToPlayer Player to send the message too. Leaving this value to its default DPID_ALLPLAYERS constant value will send the message to everyone connected to the session.
'##PD Flags Deliver flags.
'##RD Returns True if the message was sent correctly or False if an error occured.
    Dim DPMsg As DirectPlayMessage
    On Error GoTo Handler

    Set DPMsg = MEngine.DirectPlayObject.CreateMessage 'Create the message object.
    With DPMsg
        'First we detemine what type of variable the identifier is so we can write it to the message object right.
        If VarType(Identifier) = vbByte Then
            Call .WriteByte(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbDouble Then
            Call .WriteDouble(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbLong Then
            Call .WriteLong(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbInteger Then
            Call .WriteLong(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbSingle Then
            Call .WriteSingle(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbString Then
            Call .WriteSingle(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        End If
        Call .WriteDouble(Message)   'Write the double value.
    End With

    Call MEngine.DirectPlayObject.Send(mvarPlayers.MyPlayer.DPID, ToPlayer, Flags, DPMsg)
    Call MEngine.FireEvent(7, , , , , DPMsg)
    SendDouble = True

ExitIt:
    Set DPMsg = Nothing
    Exit Function

Handler:
    If Err = DPERR_ACCESSDENIED Then
        Call MEngine.FireErrorEvent(Err.Number, "The session is full, or an incorrect password was supplied.", "Session SendDouble Function")
    ElseIf Err = DPERR_CONNECTIONLOST Then
        Call MEngine.FireErrorEvent(Err.Number, "The service provider connection was reset while data was being sent.", "Session SendDouble Function")
    ElseIf Err = DPERR_INVALIDPARAMS Then
        Call MEngine.FireErrorEvent(Err.Number, "One or more of the parameters passed to the method are invalid.", "Session SendDouble Function")
    ElseIf Err = DPERR_NOTLOGGEDIN Then
        Call MEngine.FireErrorEvent(Err.Number, "An action cannot be performed because a player or client application is not logged on. Returned by the DirectPlay4.Send method when the client application tries to send a secure message without being logged on.", "Session SendDouble Function")
    ElseIf Err = DPERR_INVALIDPLAYER Then
        Call MEngine.FireErrorEvent(Err.Number, "The player ID is not recognized as a valid player ID for this game session.", "Session SendDouble Function")
    ElseIf Err = DPERR_BUSY Then
        Call MEngine.FireErrorEvent(Err.Number, "A message cannot be sent because the transmission medium is busy.", "Session SendDouble Function")
    ElseIf Err = DPERR_SENDTOOBIG Then
        Call MEngine.FireErrorEvent(Err.Number, "The message being sent by the DirectPlay4.Send method is too large.", "Session SendDouble Function")
    Else
        Call MEngine.FireErrorEvent(Err.Number, Err.Description, "Session SendDouble Function")
    End If
    SendDouble = False
    GoTo ExitIt
End Function

Public Function SendGUID(ByVal Message As Long, ByVal Identifier As Variant, Optional ByVal ToPlayer As Long = DPID_ALLPLAYERS, _
                                                                    Optional ByVal Flags As meDeliverTypes = dtDefaultDelivery) As Boolean
Attribute SendGUID.VB_HelpID = 2216
Attribute SendGUID.VB_Description = "Sends a GUID message to another Player or Group."
'##BD Sends a GUID message to another Player or Group.

'##PD Message Information to send.
'##PD Identifier Message header. Message header allows the reciever to determine what kind of message it is. Each different type of _
                            application defined message should have a unique header (identifier). Identifier value should be one of these types _
                            (Byte, Double, Long, Short, Single, String). If another value is needed use the "Send" method.
'##PD ToPlayer Player to send the message too. Leaving this value to its default DPID_ALLPLAYERS constant value will send the message to everyone connected to the session.
'##PD Flags Deliver flags.
'##RD Returns True if the message was sent correctly or False if an error occured.
    Dim DPMsg As DirectPlayMessage
    On Error GoTo Handler

    Set DPMsg = MEngine.DirectPlayObject.CreateMessage 'Create the message object.
    With DPMsg
        'First we detemine what type of variable the identifier is so we can write it to the message object right.
        If VarType(Identifier) = vbByte Then
            Call .WriteByte(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbDouble Then
            Call .WriteDouble(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbLong Then
            Call .WriteLong(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbInteger Then
            Call .WriteLong(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbSingle Then
            Call .WriteSingle(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbString Then
            Call .WriteSingle(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        End If
        Call .WriteGuid(Message)   'Write the GUID value.
    End With

    Call MEngine.DirectPlayObject.Send(mvarPlayers.MyPlayer.DPID, ToPlayer, Flags, DPMsg)
    Call MEngine.FireEvent(7, , , , , DPMsg)
    SendGUID = True

ExitIt:
    Set DPMsg = Nothing
    Exit Function

Handler:
    If Err = DPERR_ACCESSDENIED Then
        Call MEngine.FireErrorEvent(Err.Number, "The session is full, or an incorrect password was supplied.", "Session SendDouble Function")
    ElseIf Err = DPERR_CONNECTIONLOST Then
        Call MEngine.FireErrorEvent(Err.Number, "The service provider connection was reset while data was being sent.", "Session SendDouble Function")
    ElseIf Err = DPERR_INVALIDPARAMS Then
        Call MEngine.FireErrorEvent(Err.Number, "One or more of the parameters passed to the method are invalid.", "Session SendDouble Function")
    ElseIf Err = DPERR_NOTLOGGEDIN Then
        Call MEngine.FireErrorEvent(Err.Number, "An action cannot be performed because a player or client application is not logged on. Returned by the DirectPlay4.Send method when the client application tries to send a secure message without being logged on.", "Session SendDouble Function")
    ElseIf Err = DPERR_INVALIDPLAYER Then
        Call MEngine.FireErrorEvent(Err.Number, "The player ID is not recognized as a valid player ID for this game session.", "Session SendDouble Function")
    ElseIf Err = DPERR_BUSY Then
        Call MEngine.FireErrorEvent(Err.Number, "A message cannot be sent because the transmission medium is busy.", "Session SendDouble Function")
    ElseIf Err = DPERR_SENDTOOBIG Then
        Call MEngine.FireErrorEvent(Err.Number, "The message being sent by the DirectPlay4.Send method is too large.", "Session SendDouble Function")
    Else
        Call MEngine.FireErrorEvent(Err.Number, Err.Description, "Session SendDouble Function")
    End If
    SendGUID = False
    GoTo ExitIt
End Function

Public Function SendSingle(ByVal Message As Single, ByVal Identifier As Variant, Optional ByVal ToPlayer As Long = DPID_ALLPLAYERS, _
                                                                    Optional ByVal Flags As meDeliverTypes = dtDefaultDelivery) As Boolean
Attribute SendSingle.VB_HelpID = 2217
Attribute SendSingle.VB_Description = "Sends a single message to another Player or Group."
'##BD Sends a single message to another Player or Group.

'##PD Message Information to send.
'##PD Identifier Message header. Message header allows the reciever to determine what kind of message it is. Each different type of _
                            application defined message should have a unique header (identifier). Identifier value should be one of these types _
                            (Byte, Double, Long, Short, Single, String). If another value is needed use the "Send" method.
'##PD ToPlayer Player to send the message too. Leaving this value to its default DPID_ALLPLAYERS constant value will send the message to everyone connected to the session.
'##PD Flags Deliver flags.
'##RD Returns True if the message was sent correctly or False if an error occured.
    Dim DPMsg As DirectPlayMessage
    On Error GoTo Handler

    Set DPMsg = MEngine.DirectPlayObject.CreateMessage 'Create the message object.
    With DPMsg
        'First we detemine what type of variable the identifier is so we can write it to the message object right.
        If VarType(Identifier) = vbByte Then
            Call .WriteByte(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbDouble Then
            Call .WriteDouble(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbLong Then
            Call .WriteLong(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbInteger Then
            Call .WriteLong(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbSingle Then
            Call .WriteSingle(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        ElseIf VarType(Identifier) = vbString Then
            Call .WriteSingle(Identifier)  'Write the identifier value before the text to let the recieve event know what type of message it is on the other end.
        End If
        Call .WriteGuid(Message)   'Write the single value.
    End With

    Call MEngine.DirectPlayObject.Send(mvarPlayers.MyPlayer.DPID, ToPlayer, Flags, DPMsg)
    Call MEngine.FireEvent(7, , , , , DPMsg)
    SendSingle = True

ExitIt:
    Set DPMsg = Nothing
    Exit Function

Handler:
    If Err = DPERR_ACCESSDENIED Then
        Call MEngine.FireErrorEvent(Err.Number, "The session is full, or an incorrect password was supplied.", "Session SendSingle Function")
    ElseIf Err = DPERR_CONNECTIONLOST Then
        Call MEngine.FireErrorEvent(Err.Number, "The service provider connection was reset while data was being sent.", "Session SendSingle Function")
    ElseIf Err = DPERR_INVALIDPARAMS Then
        Call MEngine.FireErrorEvent(Err.Number, "One or more of the parameters passed to the method are invalid.", "Session SendSingle Function")
    ElseIf Err = DPERR_NOTLOGGEDIN Then
        Call MEngine.FireErrorEvent(Err.Number, "An action cannot be performed because a player or client application is not logged on. Returned by the DirectPlay4.Send method when the client application tries to send a secure message without being logged on.", "Session SendSingle Function")
    ElseIf Err = DPERR_INVALIDPLAYER Then
        Call MEngine.FireErrorEvent(Err.Number, "The player ID is not recognized as a valid player ID for this game session.", "Session SendSingle Function")
    ElseIf Err = DPERR_BUSY Then
        Call MEngine.FireErrorEvent(Err.Number, "A message cannot be sent because the transmission medium is busy.", "Session SendSingle Function")
    ElseIf Err = DPERR_SENDTOOBIG Then
        Call MEngine.FireErrorEvent(Err.Number, "The message being sent by the DirectPlay4.Send method is too large.", "Session SendSingle Function")
    Else
        Call MEngine.FireErrorEvent(Err.Number, Err.Description, "Session SendSingle Function")
    End If
    SendSingle = False
    GoTo ExitIt
End Function
Public Function SendAdvanced(ByVal DPMsg As DirectPlayMessage, Optional ByVal ToPlayer As Long = DPID_ALLPLAYERS, _
                                                            Optional ByVal Flags As meDeliverTypes = dtDefaultDelivery, Optional ByVal Priority As Long = 65535, _
                                                            Optional ByVal TimeOut As Long = 0, Optional ByVal Context As Long = 0) As Long
Attribute SendAdvanced.VB_HelpID = 2032
Attribute SendAdvanced.VB_Description = "Sends a DirectPlayMessage object to another Player or Group with more options."
'##BD Sends a DirectPlayMessage object to another Player or Group with more options.

'##PD DPMsg DirectPlayMessage object containing information to send.
'##PD ToPlayer Player to send the message too. Leaving this value to its default DPID_ALLPLAYERS constant value will send the message to everyone connected to the session.
'##PD Flags Deliver flags.
'##PD Priority Priority of the message.
'##PD TimeOut Time limit for delivering the message.
'##PD Context Application-defined context that is returned to the application as part of the completion message when the send has been completed.
'##RD Returns True if the message was sent correctly or False if an error occured.
    On Error GoTo Handler

    SendAdvanced = MEngine.DirectPlayObject.SendEx(mvarPlayers.MyPlayer.DPID, ToPlayer, Flags, DPMsg, Priority, _
                                                                                                    TimeOut, Context)
    Call MEngine.FireEvent(7, , , , , DPMsg)
    Exit Function

Handler:
    If Err = DPERR_ACCESSDENIED Then
        Call MEngine.FireErrorEvent(Err.Number, "The session is full, or an incorrect password was supplied.", "Session SendAdvanced Function")
    ElseIf Err = DPERR_CONNECTIONLOST Then
        Call MEngine.FireErrorEvent(Err.Number, "The service provider connection was reset while data was being sent.", "Session SendAdvanced Function")
    ElseIf Err = DPERR_INVALIDPARAMS Then
        Call MEngine.FireErrorEvent(Err.Number, "One or more of the parameters passed to the method are invalid.", "Session SendAdvanced Function")
    ElseIf Err = DPERR_NOTLOGGEDIN Then
        Call MEngine.FireErrorEvent(Err.Number, "An action cannot be performed because a player or client application is not logged on. Returned by the DirectPlay4.SendAdvanced method when the client application tries to SendAdvanced a secure message without being logged on.", "Session SendAdvanced Function")
    ElseIf Err = DPERR_INVALIDPLAYER Then
        Call MEngine.FireErrorEvent(Err.Number, "The player ID is not recognized as a valid player ID for this game session.", "Session SendAdvanced Function")
    ElseIf Err = DPERR_BUSY Then
        Call MEngine.FireErrorEvent(Err.Number, "A message cannot be sent because the transmission medium is busy.", "Session SendAdvanced Function")
    ElseIf Err = DPERR_SENDTOOBIG Then
        Call MEngine.FireErrorEvent(Err.Number, "The message being sent by the DirectPlay4.SendAdvanced method is too large.", "Session SendAdvanced Function")
    Else
        Call MEngine.FireErrorEvent(Err.Number, Err.Description, "Session SendAdvanced Function")
    End If
    SendAdvanced = 0
End Function

Public Function CancelMessage(Optional ByVal MsgID As Long = 0) As Boolean
Attribute CancelMessage.VB_HelpID = 2033
Attribute CancelMessage.VB_Description = "Cancels all or a specific message in the send queue."
'##BD Cancels all or a specific message in the send queue.

'##PD MsgID Numeric identifier of the message to cancel in the send queue. If this value is zero all messages are cancelled.
'##RD Returns True if successful and False if failed.
    On Error GoTo Handler

    Call MEngine.DirectPlayObject.CancelMessage(MsgID)
    CancelMessage = True
    Exit Function

Handler:
    If Err = DPERR_CANCELFAILED Then
        Call MEngine.FireErrorEvent(Err.Number, "The message could not be canceled, possibly because it is a group message that has already been to sent to one or more members of the group.", "Session CancelAdvancedMessage Function")
    ElseIf Err = DPERR_INVALIDFLAGS Then
        Call MEngine.FireErrorEvent(Err.Number, "The flags passed to this method are invalid.", "Session CancelAdvancedMessage Function")
    ElseIf Err = DPERR_INVALIDPARAMS Then
        Call MEngine.FireErrorEvent(Err.Number, "One or more of the parameters passed to the method are invalid.", "Session CancelAdvancedMessage Function")
    ElseIf Err = DPERR_UNKNOWNMESSAGE Then
        Call MEngine.FireErrorEvent(Err.Number, "The message ID isn't valid. Returned from DirectPlay4.CancelMessage if the ID of the message to be canceled is invalid.", "Session CancelAdvancedMessage Function")
    ElseIf Err = DPERR_UNSUPPORTED Then
        Call MEngine.FireErrorEvent(Err.Number, "The function or feature is not available in this implementation or on this service provider. Returned from DirectPlay4.SetGroupConnectionSettings if this method is called from a session that is not a lobby session. Returned from DirectPlay4.SendEx if the priority or time-out is set, and these are not supported by the service provider and DirectPlay protocol is not on. Returned from DirectPlay4.GetMessageQueue if you check the send queue and it is not supported by the service provider and DirectPlay protocol is not on. ", "Session CancelAdvancedMessage Function")
    Else
        Call MEngine.FireErrorEvent(Err.Number, Err.Description, "Session CancelAdvancedMessage Function")
    End If
    CancelMessage = False
End Function

Public Function CancelAdvancedMessagePriority(ByVal MinPriority As Long, ByVal MaxPriority As Long) As Boolean
Attribute CancelAdvancedMessagePriority.VB_HelpID = 2034
Attribute CancelAdvancedMessagePriority.VB_Description = "Cancels all messages of a particular priority or range of priorities that are still in the send queue."
'##BD Cancels all messages of a particular priority or range of priorities that are still in the send queue.

'##PD MinPriority Minimum priority in the range to be canceled.
'##PD MaxPriority Maximum priority in the range to be canceled. Must be greater than MinPriority.
    On Error GoTo Handler

    Call MEngine.DirectPlayObject.CancelPriority(MinPriority, MaxPriority)
    CancelAdvancedMessagePriority = True
    Exit Function

Handler:
    If Err = DPERR_INVALIDFLAGS Then
        Call MEngine.FireErrorEvent(Err.Number, "The flags passed to this method are invalid.", "Session CancelAdvancedMessagePriority Function")
    ElseIf Err = DPERR_INVALIDPARAMS Then
        Call MEngine.FireErrorEvent(Err.Number, "One or more of the parameters passed to the method are invalid.", "Session CancelAdvancedMessagePriority Function")
    Else
        Call MEngine.FireErrorEvent(Err.Number, Err.Description, "Session CancelAdvancedMessagePriority Function")
    End If
    CancelAdvancedMessagePriority = False
End Function
Friend Sub SetJoined()
Attribute SetJoined.VB_HelpID = 2035
    mvarJoined = True
End Sub

Public Property Let UserDefined4(ByVal vData As Long)
Attribute UserDefined4.VB_HelpID = 2036
Attribute UserDefined4.VB_Description = "Returns or sets User Defined numeric value.  "
    mvarUserDefined4 = vData
    Call mvarSessionData.SetUser4(vData)
End Property

Public Property Get UserDefined4() As Long
'##BD Returns or sets User Defined numeric value.
    UserDefined4 = mvarUserDefined4
End Property

Public Property Let UserDefined3(ByVal vData As Long)
Attribute UserDefined3.VB_HelpID = 2037
Attribute UserDefined3.VB_Description = "Returns or sets User Defined numeric value.  "
    mvarUserDefined3 = vData
    Call mvarSessionData.SetUser3(vData)
End Property

Public Property Get UserDefined3() As Long
'##BD Returns or sets User Defined numeric value.
    UserDefined3 = mvarUserDefined3
End Property

Public Property Let UserDefined2(ByVal vData As Long)
Attribute UserDefined2.VB_HelpID = 2038
Attribute UserDefined2.VB_Description = "Returns or sets User Defined numeric value.  "
    mvarUserDefined2 = vData
    Call mvarSessionData.SetUser2(vData)
End Property

Public Property Get UserDefined2() As Long
'##BD Returns or sets User Defined numeric value.
    UserDefined2 = mvarUserDefined2
End Property

Public Property Let UserDefined1(ByVal vData As Long)
Attribute UserDefined1.VB_HelpID = 2039
Attribute UserDefined1.VB_Description = "Returns or sets User Defined numeric value.  "
    mvarUserDefined1 = vData
    Call mvarSessionData.SetUser1(vData)
End Property

Public Property Get UserDefined1() As Long
'##BD Returns or sets User Defined numeric value.
    UserDefined1 = mvarUserDefined1
End Property

Public Property Let Password(ByVal vData As String)
Attribute Password.VB_HelpID = 2040
Attribute Password.VB_Description = "Returns or sets the password for the Session.  "
    mvarPassword = vData
    Call mvarSessionData.SetSessionPassword(vData)
End Property

Public Property Get Password() As String
'##BD Returns or sets the password for the Session.
    Password = mvarPassword
End Property

Public Property Let Name_(ByVal vData As String)
Attribute Name_.VB_HelpID = 2041
Attribute Name_.VB_UserMemId = 0
Attribute Name_.VB_Description = "Returns or sets the name of the Session.  "
    mvarName_ = vData
    Call mvarSessionData.SetSessionName(vData)
End Property

Public Property Get Name_() As String
'##BD Returns or sets the name of the Session.
    Name_ = mvarName_
End Property

Public Property Let MaxPlayers(ByVal vData As Long)
Attribute MaxPlayers.VB_HelpID = 2042
Attribute MaxPlayers.VB_Description = "Returns or sets the maximum number of Players that can join the Session.  "
    mvarMaxPlayers = vData
    Call mvarSessionData.SetMaxPlayers(vData)
End Property

Public Property Get MaxPlayers() As Long
'##BD Returns or sets the maximum number of Players that can join the Session.
    MaxPlayers = mvarMaxPlayers
End Property

Public Property Let GuidInstance(ByVal vData As String)
Attribute GuidInstance.VB_HelpID = 2043
Attribute GuidInstance.VB_Description = "Returns or sets the Instance GUID for the Session.  "
    mvarGuidInstance = vData
    Call mvarSessionData.SetGuidInstance(vData)
End Property

Public Property Get GuidInstance() As String
'##BD Returns or sets the Instance GUID for the Session.
    GuidInstance = mvarGuidInstance
End Property

Public Property Let GuidApplication(ByVal vData As String)
Attribute GuidApplication.VB_HelpID = 2044
Attribute GuidApplication.VB_Description = "Returns or sets the Application GUID for the Session.  "
    mvarGuidApplication = vData
    Call mvarSessionData.SetGuidApplication(vData)
End Property

Public Property Get GuidApplication() As String
'##BD Returns or sets the Application GUID for the Session.
    GuidApplication = mvarGuidApplication
End Property

Public Property Let Flags(ByVal vData As Long)
Attribute Flags.VB_HelpID = 2045
Attribute Flags.VB_Description = "Returns or sets the flags (Options) for the Session.  "
    mvarFlags = vData
    Call mvarSessionData.SetFlags(vData)
End Property

Public Property Get Flags() As Long
'##BD Returns or sets the flags (Options) for the Session.
    Flags = mvarFlags
End Property

Public Property Get CurrentPlayers() As Long
Attribute CurrentPlayers.VB_HelpID = 2046
Attribute CurrentPlayers.VB_Description = "Returns the number of Players associated with the Session."
'##BD Returns the number of Players associated with the Session.
    CurrentPlayers = mvarCurrentPlayers
End Property

Public Property Get SessionData() As DirectPlaySessionData
Attribute SessionData.VB_HelpID = 2047
Attribute SessionData.VB_Description = "Returns the Session's Data object."
'##BD Returns the Session's Data object.
    Set SessionData = mvarSessionData
End Property

Friend Sub SetInformation(SessData As DirectPlaySessionData, oParent As Sessions)
Attribute SetInformation.VB_HelpID = 2048
    On Error Resume Next

    With SessData
        mvarCurrentPlayers = .GetCurrentPlayers
        mvarFlags = .GetFlags
        mvarGuidApplication = .GetGuidApplication
        mvarGuidInstance = .GetGuidInstance
        mvarMaxPlayers = .GetMaxPlayers
        mvarName_ = .GetSessionName
        mvarPassword = .GetSessionPassword
        mvarUserDefined1 = .GetUser1
        mvarUserDefined2 = .GetUser2
        mvarUserDefined3 = .GetUser3
        mvarUserDefined4 = .GetUser4
    End With
    Set mvarParent = oParent
    Set mvarSessionData = SessData

    'Refresh the capabilities
    Call MEngine.DirectPlayObject.GetCaps(mvarCapabilities, DPGETCAPS_DEFAULT)
    Call MEngine.DirectPlayObject.GetCaps(mvarCapabilitiesGuaranteed, DPGETCAPS_GUARANTEED)
End Sub

Private Sub Class_Initialize()
    Set mvarPlayers = New Players
End Sub

Private Sub Class_Terminate()
    Set mvarPlayers = Nothing
    Set mvarParent = Nothing
End Sub


Public Property Get Capabilities() As DPCAPS
Attribute Capabilities.VB_HelpID = 2051
Attribute Capabilities.VB_Description = "Returns the capabilities of the Session."
'##BD Returns the capabilities of the Session.
    Call MEngine.DirectPlayObject.GetCaps(mvarCapabilities, DPGETCAPS_DEFAULT)
    Capabilities = mvarCapabilities
End Property

Public Property Get CapabilitiesGuaranteed() As DPCAPS
Attribute CapabilitiesGuaranteed.VB_HelpID = 2052
Attribute CapabilitiesGuaranteed.VB_Description = "Returns the guaranteed capabilities of the Session."
'##BD Returns the guaranteed capabilities of the Session.
    Call MEngine.DirectPlayObject.GetCaps(mvarCapabilitiesGuaranteed, DPGETCAPS_GUARANTEED)
    CapabilitiesGuaranteed = mvarCapabilitiesGuaranteed
End Property

Public Property Get Parent() As Sessions
Attribute Parent.VB_HelpID = 2053
Attribute Parent.VB_Description = "Returns a Sessions object directly above this object."
'##BD Returns a Sessions object directly above this object.
    Set Parent = mvarParent
End Property

Public Property Get Joined() As Boolean
Attribute Joined.VB_HelpID = 2054
Attribute Joined.VB_Description = "Returns if the Session has been joined or not."
'##BD Returns if the Session has been joined or not.
    Joined = mvarJoined
End Property
