VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Sessions"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Collection" ,"Session"
Attribute VB_Ext_KEY = "Member0" ,"Session"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
'##CD Session class is a collection of Session objects.
Attribute VB_HelpID=2055
Attribute VB_Description="Session class is a collection of Session objects."
Option Explicit

'local variable to hold collection
Private mCol As Collection
Private mvarActiveSession As Session

Friend Function Add(Key As String, SessData As DirectPlaySessionData, Optional sKey As String) As Session
Attribute Add.VB_HelpID = 2058
    'create a new object
    Dim objNewMember As Session
    Set objNewMember = New Session

    'set the properties passed into the method
    With objNewMember
        .Key = Key
        Call .SetInformation(SessData, Me)
    End With
    If Len(sKey) = 0 Then
        Call mCol.Add(objNewMember)
    Else
        Call mCol.Add(objNewMember, sKey)
    End If

    'return the object created
    Set Add = objNewMember
    Set objNewMember = Nothing
End Function

Public Function CreateSession(ByVal SessionName As String, ByVal LongName As String, ByVal ShortName As String, _
                                                        Optional ByVal MaxPlayers As Long = 10, Optional ByVal Password As String, _
                                                        Optional ByVal AppGuid As String, Optional ByVal Flags As CONST_DPSESSIONFLAGS) As Session
Attribute CreateSession.VB_HelpID = 2256
Attribute CreateSession.VB_Description = "Creates a new session."
'##BD Creates a new session.

'##PD SessionName Name of the new session.
'##PD LongName Formal name of the new player to create representing the current player.
'##PD ShortName Friendly name of the new player to create representing the current player.
'##PD MaxPlayers Total number of players allowed in this session.
'##PD Password Security clearance needed to join the session by other players.
'##PD AppGuid Application GUID. If this value is not provided a GUID is created for you.
'##PD Flags Session options. Some enum values can not be mixed together. Please check the DirectX SDK for further information. _
                        <b><i>Note:</b> If you supply a value for the Password parameter you must use the DPSESSION_PASSWORDREQUIRED _
                        enum value. DPSESSION_MIGRATEHOST enum value is also recommended for all sessions so if the host drops out the _
                        game will continue.
    Dim PlayerID As Long, Player As Player
    Dim SessionDesc As DirectPlaySessionData
    On Error GoTo Handler

    'Create the session.
    Set SessionDesc = MEngine.DirectPlayObject.CreateSessionData
    With SessionDesc
        Call .SetMaxPlayers(MaxPlayers)
        Call .SetSessionName(SessionName)
        If AppGuid = "" Then AppGuid = DirectX.CreateNewGuid 'Create one if one is not provided.
        Call .SetGuidApplication(AppGuid)
        Call .SetSessionPassword(Password)
        Call .SetFlags(Flags)
    End With
    Call MEngine.DirectPlayObject.Open(SessionDesc, DPOPEN_CREATE) 'Create a new session on the DirectPlay object.
    Set CreateSession = Add(Count + 1, SessionDesc, SessionDesc.GetGuidInstance) 'Create the new session in the collection.

    'Setup some properties
    With CreateSession
        Call .SetJoined

        'Create the player and add him to the collection.
        PlayerID = MEngine.DirectPlayObject.CreatePlayer(ShortName, LongName, NotificationID, 0)

        Call .RefreshPlayers 'Refresh the classes.
        Set mvarActiveSession = CreateSession 'Set the active session

        Call CreateSession.Players.Item(CStr(PlayerID)).SetMyPlayerFlag 'Make that player object to say this is me.
        Call MEngine.DirectPlayObject.GetSessionDesc(SessionDesc) 'Refresh the session data.
        Call .SetInformation(SessionDesc, Me)
        Call .Players.SetMyPlayer(.Players.Item(CStr(PlayerID))) 'Set the MyPlayer object.
    End With

    Call MEngine.FireEvent(6, , , , CreateSession.Players.Item(CStr(PlayerID))) 'Fire MyPlayerAdded event
    Call MEngine.FireEvent(5, , , CreateSession) 'Fire SessionJoined event

ExitIt:
    Set Player = Nothing
    Set SessionDesc = Nothing
    Exit Function

Handler:
    If Err = DPERR_INVALIDPARAMS Then
        Call MEngine.FireErrorEvent(Err.Number, "One or more of the parameters passed to the method are invalid.", "Session CreateSession Function")
    ElseIf DPERR_ACCESSDENIED Then
        Call MEngine.FireErrorEvent(Err.Number, "The session is full, or an incorrect password was supplied.", "Session CreateSession Function")
    ElseIf DPERR_ALREADYINITIALIZED Then
        Call MEngine.FireErrorEvent(Err.Number, "This object is already initialized.", "Session CreateSession Function")
    ElseIf DPERR_AUTHENTICATIONFAILED Then
        Call MEngine.FireErrorEvent(Err.Number, "The password or credentials supplied could not be authenticated.", "Session CreateSession Function")
    ElseIf DPERR_CANNOTCREATESERVER Then
        Call MEngine.FireErrorEvent(Err.Number, "The server cannot be created for the new session. ", "Session CreateSession Function")
    ElseIf DPERR_CANTCREATEPLAYER Then
        Call MEngine.FireErrorEvent(Err.Number, "A new player cannot be created.", "Session CreateSession Function")
    ElseIf DPERR_CANTLOADCAPI Then
        Call MEngine.FireErrorEvent(Err.Number, "No credentials were supplied and the CryptoAPI package (CAPI) to use for cryptography services cannot be loaded.", "Session CreateSession Function")
    ElseIf DPERR_CANTLOADSECURITYPACKAGE Then
        Call MEngine.FireErrorEvent(Err.Number, "The software security package cannot be loaded.", "Session CreateSession Function")
    ElseIf DPERR_CANTLOADSSPI Then
        Call MEngine.FireErrorEvent(Err.Number, "No credentials were supplied, and the Security Support Provider Interface (SSPI) that will prompt for credentials cannot be loaded.", "Session CreateSession Function")
    ElseIf DPERR_CONNECTING Then
        Call MEngine.FireErrorEvent(Err.Number, "The method is in the process of connecting to the network. The application should keep using the method until it returns DP_OK, indicating successful completion, or until it returns a different error.", "Session CreateSession Function")
    ElseIf DPERR_CONNECTIONLOST Then
        Call MEngine.FireErrorEvent(Err.Number, "The service provider connection was reset while data was being sent.", "Session CreateSession Function")
    ElseIf DPERR_ENCRYPTIONFAILED Then
        Call MEngine.FireErrorEvent(Err.Number, "The requested information could not be digitally encrypted. Encryption is used for message privacy. This error is only relevant in a secure session.", "Session CreateSession Function")
    ElseIf DPERR_INVALIDFLAGS Then
        Call MEngine.FireErrorEvent(Err.Number, "The flags passed to this method are invalid.", "Session CreateSession Function")
    ElseIf DPERR_INVALIDPASSWORD Then
        Call MEngine.FireErrorEvent(Err.Number, "An invalid password was supplied when attempting to CreateSession a session that requires a password.", "Session CreateSession Function")
    ElseIf DPERR_LOGONDENIED Then
        Call MEngine.FireErrorEvent(Err.Number, "The session could not be opened because credentials are required, and either no credentials were supplied, or the credentials were invalid.", "Session CreateSession Function")
    ElseIf DPERR_NOCONNECTION Then
        Call MEngine.FireErrorEvent(Err.Number, "No communication link was established.", "Session CreateSession Function")
    ElseIf DPERR_NONEWPLAYERS Then
        Call MEngine.FireErrorEvent(Err.Number, "The session is not accepting any new players.", "Session CreateSession Function")
    ElseIf DPERR_NOSESSIONS Then
        Call MEngine.FireErrorEvent(Err.Number, "There are no sessions for which this method can be called.", "Session CreateSession Function")
    ElseIf DPERR_SIGNFAILED Then
        Call MEngine.FireErrorEvent(Err.Number, "The requested information could not be digitally signed. Digital signatures are used to establish the authenticity of messages.", "Session CreateSession Function")
    ElseIf DPERR_TIMEOUT Then
        Call MEngine.FireErrorEvent(Err.Number, "The operation could not be completed in the specified time.", "Session CreateSession Function")
    ElseIf DPERR_UNINITIALIZED Then
        Call MEngine.FireErrorEvent(Err.Number, "The requested object has not been initialized.", "Session CreateSession Function")
    ElseIf DPERR_USERCANCEL Then
        Call MEngine.FireErrorEvent(Err.Number, "Can be returned in two ways. 1) The user canceled the connection process during a call to the DirectPlay4.Open method. 2) The user clicked Cancel in one of the DirectPlay service provider dialog boxes during a call to DirectPlay4.GetDPEnumSessions.", "Session CreateSession Function")
    ElseIf DPERR_CANTADDPLAYER Then
        Call MEngine.FireErrorEvent(Err.Number, "The player cannot be added to the session.", "Session CreateSession Function")
    ElseIf DPERR_CANTCREATEPLAYER Then
        Call MEngine.FireErrorEvent(Err.Number, "A new player cannot be created.", "Session CreateSession Function")
    Else
        Call MEngine.FireErrorEvent(Err.Number, Err.Description, "Session CreateSession Function")
    End If
    GoTo ExitIt
End Function

Public Function CreateSecuredSession(ByVal SessionName As String, ByVal LongName As String, ByVal ShortName As String, _
                                                                        Security As DPSECURITYDESC, Optional ByVal MaxPlayers As Long = 10, _
                                                                        Optional ByVal Password As String, Optional ByVal AppGuid As String, _
                                                                        Optional ByVal Flags As CONST_DPSESSIONFLAGS) As Session
Attribute CreateSecuredSession.VB_HelpID = 2257
Attribute CreateSecuredSession.VB_Description = "Creates a new secured session."
'##BD Creates a new secured session.

'##PD SessionDesc DirectPlaySessionData object used to describe the capabilities of the new Session.
'##PD LongName Formal name of the new player to create representing the current player.
'##PD ShortName Friendly name of the new player to create representing the current player.
'##PD Security DPSECURITYDESC type containing the security package to use.
    Dim PlayerID As Long, Player As Player, Credentials As DPCREDENTIALS
    Dim SessionDesc As DirectPlaySessionData
    On Error GoTo Handler

    'Create the session.
    Set SessionDesc = MEngine.DirectPlayObject.CreateSessionData
    With SessionDesc
        Call .SetMaxPlayers(MaxPlayers)
        Call .SetSessionName(SessionName)
        If AppGuid = "" Then AppGuid = DirectX.CreateNewGuid 'Create one if one is not provided.
        Call .SetGuidApplication(AppGuid)
        Call .SetSessionPassword(Password)
        Call .SetFlags(Flags)
    End With
    With Credentials 'Set the credentials type members to 0. When creating a session the credentials is actually ignored.
        .lFlags = 0:        .strDomain = 0:        .strPassword = 0:        .strUsername = 0
    End With
    Call MEngine.DirectPlayObject.SecureOpen(SessionDesc, DPOPEN_CREATE, Security, Credentials) 'Create a new session on the DirectPlay object.
    Set CreateSecuredSession = Add(Count + 1, SessionDesc, SessionDesc.GetGuidInstance) 'Create the new session in the collection.

    'Setup some properties
    With CreateSecuredSession
        Call .SetJoined

        'Create the player and add him to the collection.
        PlayerID = MEngine.DirectPlayObject.CreatePlayer(ShortName, LongName, NotificationID, 0)

        Call .RefreshPlayers 'Refresh the classes.
        Set mvarActiveSession = CreateSecuredSession 'Set the active session

        Call CreateSecuredSession.Players.Item(CStr(PlayerID)).SetMyPlayerFlag 'Make that player object to say this is me.
        Call MEngine.DirectPlayObject.GetSessionDesc(SessionDesc) 'Refresh the session data.
        Call .SetInformation(SessionDesc, Me)
        Call .Players.SetMyPlayer(.Players.Item(CStr(PlayerID))) 'Set the MyPlayer object.
    End With

    Call MEngine.FireEvent(6, , , , CreateSecuredSession.Players.Item(CStr(PlayerID))) 'Fire MyPlayerAdded event
    Call MEngine.FireEvent(5, , , CreateSecuredSession) 'Fire SessionJoined event

ExitIt:
    Set Player = Nothing
    Set SessionDesc = Nothing
    Exit Function

Handler:
    If Err = DPERR_INVALIDPARAMS Then
        Call MEngine.FireErrorEvent(Err.Number, "One or more of the parameters passed to the method are invalid.", "Sessions CreateSecuredSession Function")
    ElseIf DPERR_ACCESSDENIED Then
        Call MEngine.FireErrorEvent(Err.Number, "The session is full, or an incorrect password was supplied.", "Sessions CreateSecuredSession Function")
    ElseIf DPERR_ALREADYINITIALIZED Then
        Call MEngine.FireErrorEvent(Err.Number, "This object is already initialized.", "Sessions CreateSecuredSession Function")
    ElseIf DPERR_AUTHENTICATIONFAILED Then
        Call MEngine.FireErrorEvent(Err.Number, "The password or credentials supplied could not be authenticated.", "Sessions CreateSecuredSession Function")
    ElseIf DPERR_CANNOTCREATESERVER Then
        Call MEngine.FireErrorEvent(Err.Number, "The server cannot be created for the new session. ", "Sessions CreateSecuredSession Function")
    ElseIf DPERR_CANTCREATEPLAYER Then
        Call MEngine.FireErrorEvent(Err.Number, "A new player cannot be created.", "Sessions CreateSecuredSession Function")
    ElseIf DPERR_CANTLOADCAPI Then
        Call MEngine.FireErrorEvent(Err.Number, "No credentials were supplied and the CryptoAPI package (CAPI) to use for cryptography services cannot be loaded.", "Sessions CreateSecuredSession Function")
    ElseIf DPERR_CANTLOADSECURITYPACKAGE Then
        Call MEngine.FireErrorEvent(Err.Number, "The software security package cannot be loaded.", "Sessions CreateSecuredSession Function")
    ElseIf DPERR_CANTLOADSSPI Then
        Call MEngine.FireErrorEvent(Err.Number, "No credentials were supplied, and the Security Support Provider Interface (SSPI) that will prompt for credentials cannot be loaded.", "Sessions CreateSecuredSession Function")
    ElseIf DPERR_CONNECTING Then
        Call MEngine.FireErrorEvent(Err.Number, "The method is in the process of connecting to the network. The application should keep using the method until it returns DP_OK, indicating successful completion, or until it returns a different error.", "Sessions CreateSecuredSession Function")
    ElseIf DPERR_CONNECTIONLOST Then
        Call MEngine.FireErrorEvent(Err.Number, "The service provider connection was reset while data was being sent.", "Sessions CreateSecuredSession Function")
    ElseIf DPERR_ENCRYPTIONFAILED Then
        Call MEngine.FireErrorEvent(Err.Number, "The requested information could not be digitally encrypted. Encryption is used for message privacy. This error is only relevant in a secure session.", "Sessions CreateSecuredSession Function")
    ElseIf DPERR_INVALIDFLAGS Then
        Call MEngine.FireErrorEvent(Err.Number, "The flags passed to this method are invalid.", "Sessions CreateSecuredSession Function")
    ElseIf DPERR_INVALIDPASSWORD Then
        Call MEngine.FireErrorEvent(Err.Number, "An invalid password was supplied when attempting to CreateSecuredSession a session that requires a password.", "Sessions CreateSecuredSession Function")
    ElseIf DPERR_LOGONDENIED Then
        Call MEngine.FireErrorEvent(Err.Number, "The session could not be opened because credentials are required, and either no credentials were supplied, or the credentials were invalid.", "Sessions CreateSecuredSession Function")
    ElseIf DPERR_NOCONNECTION Then
        Call MEngine.FireErrorEvent(Err.Number, "No communication link was established.", "Sessions CreateSecuredSession Function")
    ElseIf DPERR_NONEWPLAYERS Then
        Call MEngine.FireErrorEvent(Err.Number, "The session is not accepting any new players.", "Sessions CreateSecuredSession Function")
    ElseIf DPERR_NOSESSIONS Then
        Call MEngine.FireErrorEvent(Err.Number, "There are no sessions for which this method can be called.", "Sessions CreateSecuredSession Function")
    ElseIf DPERR_SIGNFAILED Then
        Call MEngine.FireErrorEvent(Err.Number, "The requested information could not be digitally signed. Digital signatures are used to establish the authenticity of messages.", "Sessions CreateSecuredSession Function")
    ElseIf DPERR_TIMEOUT Then
        Call MEngine.FireErrorEvent(Err.Number, "The operation could not be completed in the specified time.", "Sessions CreateSecuredSession Function")
    ElseIf DPERR_UNINITIALIZED Then
        Call MEngine.FireErrorEvent(Err.Number, "The requested object has not been initialized.", "Sessions CreateSecuredSession Function")
    ElseIf DPERR_USERCANCEL Then
        Call MEngine.FireErrorEvent(Err.Number, "Can be returned in two ways. 1) The user canceled the connection process during a call to the DirectPlay4.Open method. 2) The user clicked Cancel in one of the DirectPlay service provider dialog boxes during a call to DirectPlay4.GetDPEnumSessions.", "Sessions CreateSecuredSession Function")
    ElseIf DPERR_CANTADDPLAYER Then
        Call MEngine.FireErrorEvent(Err.Number, "The player cannot be added to the session.", "Sessions CreateSecuredSession Function")
    ElseIf DPERR_CANTCREATEPLAYER Then
        Call MEngine.FireErrorEvent(Err.Number, "A new player cannot be created.", "Sessions CreateSecuredSession Function")
    Else
        Call MEngine.FireErrorEvent(Err.Number, Err.Description, "Sessions CreateSecuredSession Function")
    End If
    GoTo ExitIt
End Function
Public Property Get Item(vntIndexKey As Variant) As Session
Attribute Item.VB_HelpID = 2061
Attribute Item.VB_UserMemId = 0
    Set Item = mCol(vntIndexKey)
End Property



Public Property Get Count() As Long
Attribute Count.VB_HelpID = 2062
Attribute Count.VB_Description = "Returns the number of Session objects located within collection."
'##BD Returns the number of Session objects located within collection.
    Count = mCol.Count
End Property


Friend Sub KillActiveSession()
Attribute KillActiveSession.VB_HelpID = 2063
    Set mvarActiveSession = Nothing
End Sub

Friend Sub Remove(vntIndexKey As Variant)
Attribute Remove.VB_HelpID = 2064
    Call mCol.Remove(vntIndexKey)
End Sub

Public Property Get NewEnum() As IUnknown
Attribute NewEnum.VB_HelpID = 2065
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
    Set NewEnum = mCol.[_NewEnum]
End Property


Friend Sub SetActiveSession(Session As Session)
Attribute SetActiveSession.VB_HelpID = 2066
    Set mvarActiveSession = Session
End Sub

Private Sub Class_Initialize()
    'creates the collection when this class is created
    Set mCol = New Collection
End Sub


Private Sub Class_Terminate()
    'destroys collection when this class is terminated
    Set mCol = Nothing
End Sub

Friend Sub Clear()
Attribute Clear.VB_HelpID = 2069
    Do Until Count = 0
        Call Remove(1)
    Loop
End Sub

Public Property Get ActiveSession() As Session
Attribute ActiveSession.VB_HelpID = 2070
Attribute ActiveSession.VB_Description = "Returns the Session object representing the Session that was joined."
'##BD Returns the Session object representing the Session that was joined.
    Set ActiveSession = mvarActiveSession
End Property
